@@ -19,7 +19,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sg_time.cxx,v 1.31 2002/02/11 22:27:28 curt Exp $
+// $Id: sg_time.cxx,v 1.5 2003/06/09 20:19:54 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,8 @@
 #  include <stdlib.h>
 #endif
 
+#include <string>
+
 #ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>  // for get/setitimer, gettimeofday, struct timeval
 #endif
@@ -60,7 +62,6 @@
 #include "timezone.h"
 #include "lowleveltime.h"
 
-
 #define DEGHR(x)        ((x)/15.)
 #define RADHR(x)        DEGHR(x*SGD_RADIANS_TO_DEGREES)
 
@@ -70,7 +71,7 @@ static const double J2000   = 2451545.0 - MJD0;
 static const double SIDRATE = 0.9972695677;
 
 
-void SGTime::init( double lon, double lat,
+void SGTime::init( double lon_rad, double lat_rad,
                    const string& root, time_t init_time )
 {
     SG_LOG( SG_EVENT, SG_INFO, "Initializing Time" );
@@ -83,35 +84,36 @@ void SGTime::init( double lon, double lat,
 	cur_time = time(NULL); 
     }
 
-    cout << "Current greenwich mean time = " << asctime(gmtime(&cur_time))
-         << endl;
-    cout << "Current local time          = " 
-         << asctime(localtime(&cur_time)) << endl;
+    SG_LOG( SG_EVENT, SG_INFO,
+                "Current greenwich mean time = " << asctime(gmtime(&cur_time)));
+    SG_LOG( SG_EVENT, SG_INFO,
+             "Current local time          = " << asctime(localtime(&cur_time)));
 
-    if ( root != (string)"" ) {
+    if ( !root.empty()) {
         SGPath zone( root );
         zone.append( "zone.tab" );
         SG_LOG( SG_EVENT, SG_INFO, "Reading timezone info from: "
                 << zone.str() );
-        tzContainer = new TimezoneContainer( zone.c_str() );
+        tzContainer = new SGTimeZoneContainer( zone.c_str() );
 
-        GeoCoord location( SGD_RADIANS_TO_DEGREES * lat, SGD_RADIANS_TO_DEGREES * lon );
-        GeoCoord* nearestTz = tzContainer->getNearest(location);
+        SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat_rad, SGD_RADIANS_TO_DEGREES * lon_rad );
+        SGGeoCoord* nearestTz = tzContainer->getNearest(location);
 
         SGPath name( root );
         name.append( nearestTz->getDescription() );
-        zonename = strdup( name.c_str() );
+        zonename = name.str();
         SG_LOG( SG_EVENT, SG_INFO, "Using zonename = " << zonename );
     } else {
         SG_LOG( SG_EVENT, SG_INFO, "*** NO TIME ZONE NAME ***" );
         tzContainer = NULL;
-        zonename = NULL;
+        zonename.erase();
     }
 }
 
-SGTime::SGTime( double lon, double lat, const string& root, time_t init_time )
+SGTime::SGTime( double lon_rad, double lat_rad, const string& root,
+                time_t init_time )
 {
-    init( lon, lat, root, init_time );
+    init( lon_rad, lat_rad, root, init_time );
 }
 
 
@@ -128,16 +130,10 @@ SGTime::SGTime() {
 SGTime::~SGTime()
 {
     if ( tzContainer != NULL ) {
-        TimezoneContainer *tmp = tzContainer;
+        SGTimeZoneContainer *tmp = tzContainer;
         tzContainer = NULL;
 	delete tmp;
     }
-
-    if ( zonename != NULL ) {
-        char *tmp = zonename;
-        zonename = NULL;
-	delete tmp;
-    }
 }
 
 
@@ -201,7 +197,9 @@ static double sidereal_course( time_t cur_time, const struct tm *gmt, double lng
 
 
 // Update the time related variables
-void SGTime::update( double lon, double lat, time_t ct, long int warp ) {
+void SGTime::update( double lon_rad, double lat_rad,
+                     time_t ct, long int warp )
+{
     double gst_precise, gst_course;
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
@@ -256,60 +254,60 @@ void SGTime::update( double lon, double lat, time_t ct, long int warp ) {
       
 	gst_diff = gst_precise - gst_course;
 
-	lst = sidereal_course( cur_time, gmt, -(lon * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
+	lst = sidereal_course( cur_time, gmt,
+                               -(lon_rad * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
     } else {
 	// course + difference should drift off very slowly
 	gst = sidereal_course( cur_time, gmt, 0.00 ) + gst_diff;
-	lst = sidereal_course( cur_time, gmt, -(lon * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
+	lst = sidereal_course( cur_time, gmt,
+                               -(lon_rad * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
     }
 
     SG_LOG( SG_EVENT, SG_DEBUG,
 	    "  Current lon=0.00 Sidereal Time = " << gst );
     SG_LOG( SG_EVENT, SG_DEBUG,
 	    "  Current LOCAL Sidereal Time = " << lst << " (" 
-	    << sidereal_precise( mjd, -(lon * SGD_RADIANS_TO_DEGREES) ) 
+	    << sidereal_precise( mjd, -(lon_rad * SGD_RADIANS_TO_DEGREES) ) 
 	    << ") (diff = " << gst_diff << ")" );
 }
 
 
 // Given lon/lat, update timezone information and local_offset
-void SGTime::updateLocal( double lon, double lat, const string& root ) {
+void SGTime::updateLocal( double lon_rad, double lat_rad, const string& root ) {
     // sanity checking
-    if ( lon < -SGD_PI || lon > SGD_PI ) {
+    if ( lon_rad < -SGD_PI || lon_rad> SGD_PI ) {
         // not within -180 ... 180
-        lon = 0.0;
+        lon_rad = 0.0;
     }
-    if ( lat < -SGD_PI * 0.5 || lat > SGD_PI * 0.5 ) {
+    if ( lat_rad < -SGD_PI * 0.5 || lat_rad > SGD_PI * 0.5 ) {
         // not within -90 ... 90
-        lat = 0.0;
+        lat_rad = 0.0;
     }
-    if ( lon != lon ) {
-        // only true if lon == nan
-        SG_LOG( SG_EVENT, SG_ALERT, "  Detected lon == nan, resetting to 0.0" );
-        lon = 0.0;
+    if ( lon_rad != lon_rad ) {
+        // only true if lon_rad == nan
+        SG_LOG( SG_EVENT, SG_ALERT,
+                "  Detected lon_rad == nan, resetting to 0.0" );
+        lon_rad = 0.0;
     }
-    if ( lat != lat ) {
-        // only true if lat == nan
-        SG_LOG( SG_EVENT, SG_ALERT, "  Detected lat == nan, resetting to 0.0" );
-        lat = 0.0;
+    if ( lat_rad != lat_rad ) {
+        // only true if lat_rad == nan
+        SG_LOG( SG_EVENT, SG_ALERT,
+                "  Detected lat_rad == nan, resetting to 0.0" );
+        lat_rad = 0.0;
     }
     time_t currGMT;
     time_t aircraftLocalTime;
-    GeoCoord location( SGD_RADIANS_TO_DEGREES * lat, SGD_RADIANS_TO_DEGREES * lon );
-    GeoCoord* nearestTz = tzContainer->getNearest(location);
+    SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat_rad,
+                         SGD_RADIANS_TO_DEGREES * lon_rad );
+    SGGeoCoord* nearestTz = tzContainer->getNearest(location);
     SGPath zone( root );
     zone.append ( nearestTz->getDescription() );
-    if ( zonename ) {
-        char *ptr = zonename;
-        zonename = NULL;
-        delete ptr;
-    }
-    zonename = strdup( zone.c_str() );
+    zonename = zone.str();
 
     //Avoid troubles when zone.tab hasn't got the right line endings
-    if (zonename[strlen(zonename)-1] == '\r')
+    if (zonename[zonename.size()-1] == '\r')
     {
-      zonename[strlen(zonename)-1]=0;
+      zonename[zonename.size()-1]=0;
       zone.set( zonename );
     }
 
